from django.templatetags.static import static
from django.utils.html import format_html_join

from wagtail import hooks

from .settings import get_theme, HLJS_VERSION, HLJS_PREFIX


@hooks.register("insert_global_admin_css")
def global_admin_css():
    THEME = get_theme()

    if THEME:
        hljs_theme = f"{THEME}"
    else:
        hljs_theme = ""

    extra_css = [
        f"https:{HLJS_PREFIX}/{HLJS_VERSION}/styles/{hljs_theme}.min.css",
        static("wagtail_hljs/css/wagtail-hljs-block.min.css"),
    ]

    return format_html_join(
        "\n",
        '<link rel="stylesheet" style="text/css" href="{}">',
        ((f,) for f in extra_css),
    )


@hooks.register("insert_global_admin_js")
def global_admin_js():
    js_files = [
        f"{HLJS_PREFIX}/{HLJS_VERSION}/highlight.min.js",
    ]

    js_includes = format_html_join(
        "\n",
        """<script type="text/javascript" src="{}"></script>""",
        ((f,) for f in js_files),
    )

    return js_includes



from django.conf import settings

HLJS_PREFIX = "//cdnjs.cloudflare.com/ajax/libs/highlight.js"
HLJS_VERSION = "11.9.0"


def get_language_choices():
    """
    Default list of language choices, if not overridden by Django.
    """
    DEFAULT_LANGUAGES = (
        ("bash", "Bash/Shell"),
        ("css", "CSS"),
        ("diff", "diff"),
        ("html", "HTML"),
        ("javascript", "Javascript"),
        ("json", "JSON"),
        ("python", "Python"),
        ("scss", "SCSS"),
        ("yaml", "YAML"),
    )

    return getattr(settings, "WAGTAIL_CODE_BLOCK_LANGUAGES", DEFAULT_LANGUAGES)


def get_theme():
    """
    Returns a default theme, if not in the proejct's settings. Default theme is 'coy'.
    """

    return getattr(settings, "WAGTAIL_CODE_BLOCK_THEME", "coy")


def get_line_numbers():
    """
    Returns the line numbers setting.
    """

    return getattr(settings, "WAGTAIL_CODE_BLOCK_LINE_NUMBERS", True)


def get_copy_to_clipboard():
    """
    Returns the copy to clipboard setting.
    """

    return getattr(settings, "WAGTAIL_CODE_BLOCK_COPY_TO_CLIPBOARD", True)



from django.forms import Media
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _

from wagtail.blocks import (
    StructBlock,
    TextBlock,
    ChoiceBlock,
)
from wagtail.blocks.struct_block import StructBlockAdapter
from wagtail.telepath import register

from .settings import get_language_choices


class CodeBlock(StructBlock):
    """
    A Wagtail StreamField block for code syntax highlighting using PrismJS.
    """

    def __init__(self, local_blocks=None, **kwargs):
        # Languages included in PrismJS core
        # Review: https://github.com/PrismJS/prism/blob/gh-pages/prism.js#L602
        self.INCLUDED_LANGUAGES = (
            ("html", "HTML"),
            ("mathml", "MathML"),
            ("svg", "SVG"),
            ("xml", "XML"),
        )

        if local_blocks is None:
            local_blocks = []
        else:
            local_blocks = local_blocks.copy()

        language_choices, language_default = self.get_language_choice_list(**kwargs)

        local_blocks.extend(
            [
                (
                    "language",
                    ChoiceBlock(
                        choices=language_choices,
                        help_text=_("Coding language"),
                        label=_("Language"),
                        default=language_default,
                        identifier="language",
                    ),
                ),
                ("code", TextBlock(label=_("Code"), identifier="code")),
            ]
        )

        super().__init__(local_blocks, **kwargs)

    def get_language_choice_list(self, **kwargs):
        # Get default languages
        WCB_LANGUAGES = get_language_choices()
        # If a language is passed in as part of a code block, use it.
        language = kwargs.get("language", False)

        total_language_choices = WCB_LANGUAGES + self.INCLUDED_LANGUAGES

        if language in [lang[0] for lang in total_language_choices]:
            for language_choice in total_language_choices:
                if language_choice[0] == language:
                    language_choices = (language_choice,)
                    language_default = language_choice[0]
        else:
            language_choices = WCB_LANGUAGES
            language_default = kwargs.get("default_language")

        return language_choices, language_default

    class Meta:
        icon = "code"
        template = "wagtail_hljs/code_block.html"
        form_classname = "code-block struct-block"
        form_template = "wagtail_hljs/code_block_form.html"


class CodeBlockAdapter(StructBlockAdapter):
    js_constructor = "wagtail_hljs.blocks.CodeBlock"

    @cached_property
    def media(self):
        structblock_media = super().media
        return Media(
            js=structblock_media._js + ["wagtail_hljs/js/wagtailhljsblock.js"],
            css=structblock_media._css,
        )


register(CodeBlockAdapter(), CodeBlock)



from django.template import Library
from django.utils.safestring import mark_safe

from ..settings import get_theme, HLJS_VERSION, HLJS_PREFIX

register = Library()


@register.simple_tag
def hljs_prefix():
    return HLJS_PREFIX


@register.simple_tag
def hljs_version():
    """Returns the version of Highlight.js."""

    return HLJS_VERSION


@register.simple_tag
def load_hljs_theme():
    """Loads the Highlight.js theme."""
    theme = get_theme()

    if theme:
        css_link = f"""<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/highlight.js/{HLJS_VERSION}/styles/{theme}.min.css">"""
    else:
        css_link = f"""<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/highlight.js/{HLJS_VERSION}/styles/default.min.css">"""

    return mark_safe(css_link)
